SystemOrganization addCategory: #'Collections-OrderPreservingDictionary'!

Collection subclass: #StandardOrderedDictionary
	instanceVariableNames: 'dictionary orderedKeys'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-OrderPreservingDictionary'!

!StandardOrderedDictionary commentStamp: 'monty 11/21/2014 03:57' prior: 0!
This class is a dictionary that uses key insertion order when enumerating, printing, or returing collections of keys/values/associations, but not when testing for equality.

Insertion, updating, and inclusion testing have O(1) complexity while removing has O(n) worst-case.!

StandardOrderedDictionary subclass: #OrderPreservingDictionary
	instanceVariableNames: 'defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-OrderPreservingDictionary'!

!OrderPreservingDictionary commentStamp: 'monty 11/21/2014 03:51' prior: 0!
An order-preserving dictionary that returns a configurable default value (nil by default) when an absent key or value is requested from it rather than raising an exception.!

!OrderPreservingDictionary class methodsFor: 'instance creation' stamp: 'JAAyer 12/5/2010 11:21'!
defaultValue: aDefaultValue
	^ self new defaultValue: aDefaultValue! !

!OrderPreservingDictionary class methodsFor: 'instance creation' stamp: 'JAAyer 12/4/2010 20:36'!
new: aCapacity withDefaultValue: aDefaultValue
	^ self basicNew initialize: aCapacity withDefaultValue: aDefaultValue! !

!OrderPreservingDictionary class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 18:23'!
newFrom: anAssociationCollection
	| newDictionary |

	newDictionary := super newFrom: anAssociationCollection.
	(anAssociationCollection respondsTo: #defaultValue)
		ifTrue: [newDictionary defaultValue: anAssociationCollection defaultValue].
	^ newDictionary.! !

!OrderPreservingDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:06'!
associationAt: aKey
	^ self associationAt: aKey ifAbsent: [nil]! !

!OrderPreservingDictionary methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 20:39'!
at: aKey
	^ self at: aKey ifAbsent: [defaultValue]! !

!OrderPreservingDictionary methodsFor: 'copying' stamp: 'JAAyer 12/5/2010 11:25'!
copyEmpty
	^ self species defaultValue: defaultValue! !

!OrderPreservingDictionary methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 20:39'!
defaultValue
	^ defaultValue! !

!OrderPreservingDictionary methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 20:39'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!OrderPreservingDictionary methodsFor: 'initialization' stamp: 'monty 11/20/2014 18:22'!
initialize: aCapacity withDefaultValue: aDefaultValue
	self initialize: aCapacity.
	defaultValue := aDefaultValue.! !

!OrderPreservingDictionary methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 20:40'!
keyAtIdentityValue: aValue
	^ self keyAtIdentityValue: aValue ifAbsent: [defaultValue]! !

!OrderPreservingDictionary methodsFor: 'accessing' stamp: 'monty 11/3/2014 16:56'!
keyAtIndex: anIndex
	^ self keyAtIndex: anIndex ifAbsent: [defaultValue]! !

!OrderPreservingDictionary methodsFor: 'accessing' stamp: 'JAAyer 12/4/2010 20:40'!
keyAtValue: aValue
	^ self keyAtValue: aValue ifAbsent: [defaultValue]! !

!OrderPreservingDictionary methodsFor: 'removing' stamp: 'JAAyer 12/4/2010 20:40'!
removeKey: aKey
	^ self removeKey: aKey ifAbsent: [defaultValue]! !

!OrderPreservingDictionary methodsFor: 'private' stamp: 'monty 11/14/2014 16:27'!
speciesNewFrom: anAssociationCollection
	^ (self species newFrom: anAssociationCollection)
		defaultValue: defaultValue! !

OrderPreservingDictionary subclass: #OrderPreservingIdentityDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-OrderPreservingDictionary'!

!OrderPreservingIdentityDictionary commentStamp: 'monty 11/21/2014 03:46' prior: 0!
An IdentityDictionary version that uses == instead of = for key comparing.!

!OrderPreservingIdentityDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:27'!
dictionaryClass
	^ IdentityDictionary! !

!OrderPreservingIdentityDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:35'!
indexOfKey: aKey ifAbsent: aBlock
	^ self identityIndexOfKey: aKey ifAbsent: aBlock! !

StandardOrderedDictionary subclass: #OrderPreservingStringDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-OrderPreservingDictionary'!

!OrderPreservingStringDictionary commentStamp: 'monty 11/21/2014 03:45' prior: 0!
An order-preserving dictionary for strings that returns empty strings instead of raising errors when absent keys/values are accessed.!

OrderPreservingStringDictionary subclass: #OrderPreservingIdentityStringDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-OrderPreservingDictionary'!

!OrderPreservingIdentityStringDictionary commentStamp: 'monty 11/21/2014 03:46' prior: 0!
An IdentityDictionary version that uses == instead of = for key comparing.!

!OrderPreservingIdentityStringDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:31'!
dictionaryClass
	^ IdentityDictionary! !

!OrderPreservingIdentityStringDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:35'!
indexOfKey: aKey ifAbsent: aBlock
	^ self identityIndexOfKey: aKey ifAbsent: aBlock! !

!OrderPreservingStringDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:40'!
associationAt: aKey
	^ self associationAt: aKey ifAbsent: [nil]! !

!OrderPreservingStringDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:18'!
at: aKey
	^ self at: aKey ifAbsent: ['']! !

!OrderPreservingStringDictionary methodsFor: 'accessing' stamp: 'monty 11/21/2014 04:42'!
defaultValue
	^ ''! !

!OrderPreservingStringDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:18'!
keyAtIdentityValue: aValue
	^ self keyAtIdentityValue: aValue ifAbsent: ['']! !

!OrderPreservingStringDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:19'!
keyAtIndex: anIndex
	^ self keyAtIndex: anIndex ifAbsent: ['']! !

!OrderPreservingStringDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:20'!
keyAtValue: aValue
	^ self keyAtValue: aValue ifAbsent: ['']! !

!OrderPreservingStringDictionary methodsFor: 'removing' stamp: 'monty 11/20/2014 18:19'!
removeKey: aKey
	^ self removeKey: aKey ifAbsent: ['']! !

!StandardOrderedDictionary class methodsFor: 'inspecting' stamp: 'monty 11/20/2014 18:12'!
inspectorClass
	^ Dictionary inspectorClass! !

!StandardOrderedDictionary class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 18:12'!
new
	^ self new: 10! !

!StandardOrderedDictionary class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 18:21'!
new: aCapacity
	^ self basicNew initialize: aCapacity! !

!StandardOrderedDictionary class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 18:22'!
newFrom: anAssociationCollection
	| newDictionary |

	newDictionary := self new: anAssociationCollection size.
	anAssociationCollection associationsDo: [:each |
		newDictionary
			at: each key
			put: each value].
	^ newDictionary.! !

!StandardOrderedDictionary class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 18:12'!
newFromPairs: aSequenceableCollection
	| newDictionary |

	newDictionary := self new: (aSequenceableCollection size / 2) floor.
	1 to: aSequenceableCollection size - 1 by: 2 do: [:i |
		newDictionary
			at: (aSequenceableCollection at: i)
			put: (aSequenceableCollection at: i + 1)].
	^ newDictionary.! !

!StandardOrderedDictionary methodsFor: 'comparing' stamp: 'monty 11/20/2014 18:03'!
= anObject
	self == anObject
		ifTrue: [^ true].

	(anObject isDictionary and: [anObject size = self size])
		ifFalse: [^ false].

	dictionary associationsDo: [:each |
		(anObject at: each key ifAbsent: [^ false]) = each value
			ifFalse: [^ false]].
	^ true.! !

!StandardOrderedDictionary methodsFor: 'adding' stamp: 'monty 11/20/2014 18:03'!
add: anAssociation
	| oldSize newSize |

	oldSize := dictionary size.
	dictionary add: anAssociation.
	(newSize := dictionary size) > oldSize
		ifTrue: [
			 newSize > orderedKeys size
				ifTrue: [self growOrderedKeys].
			orderedKeys at: newSize put: anAssociation key].
	^ anAssociation.! !

!StandardOrderedDictionary methodsFor: 'adding' stamp: 'monty 11/21/2014 04:03'!
addAll: anAssociationCollection
	"Since Collection implements #associationsDo, this method can accept any collection
	of associations including Arrays and OrderedCollections and not just Dictionaries"

	anAssociationCollection associationsDo: [:each | self add: each].
	^ anAssociationCollection.! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/21/2014 01:38'!
associationAt: aKey
	^ dictionary associationAt: aKey! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:06'!
associationAt: aKey ifAbsent: aBlock
	^ dictionary associationAt: aKey ifAbsent: aBlock! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:06'!
associationAt: aKey ifPresent: aBlock
	"squeak does not have associationAt:ifPresent:"
	^ aBlock cull: (dictionary associationAt: aKey ifAbsent: [^ nil])! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:07'!
associations
	| associations i |

	associations := Array new: self size.
	i := 1.
	self associationsDo: [:each |
		associations at: i put: each.
		i := i + 1].
	^ associations.! !

!StandardOrderedDictionary methodsFor: 'enumerating' stamp: 'monty 11/20/2014 18:07'!
associationsDo: aBlock
	self keysDo: [:each | aBlock value: (self associationAt: each)]! !

!StandardOrderedDictionary methodsFor: 'enumerating' stamp: 'monty 11/20/2014 18:07'!
associationsSelect: aBlock
	^ self speciesNewFrom: (self associations select: aBlock)! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/21/2014 01:38'!
at: aKey
	^ dictionary at: aKey! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:06'!
at: aKey ifAbsent: aBlock
	^ dictionary at: aKey ifAbsent: aBlock! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:07'!
at: aKey ifAbsentPut: aBlock
	^ self at: aKey ifAbsent: [self at: aKey put: aBlock value]! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:07'!
at: aKey ifPresent: aBlock
	^ dictionary at: aKey ifPresent: aBlock! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:07'!
at: aKey ifPresent: aPresentBlock ifAbsent: anAbsentBlock
	^ dictionary
		at: aKey
		ifPresent: aPresentBlock
		ifAbsent: anAbsentBlock! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:07'!
at: aKey put: aValue
	| oldSize newSize |

	oldSize := dictionary size.
	dictionary at: aKey put: aValue.
	(newSize := dictionary size) > oldSize
		ifTrue: [
			 newSize > orderedKeys size
				ifTrue: [self growOrderedKeys].
			orderedKeys at: newSize put: aKey].
	^ aValue.! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:07'!
capacity
	^ dictionary capacity! !

!StandardOrderedDictionary methodsFor: 'enumerating' stamp: 'monty 11/20/2014 18:07'!
collect: aBlock
	^ self speciesNewFrom:
		(self associations collect: [:each |
			each key -> (aBlock value: each value)])! !

!StandardOrderedDictionary methodsFor: 'copying' stamp: 'monty 12/2/2014 06:21'!
copyEmpty
	"for compatibility with squeak"
	^ self species new! !

!StandardOrderedDictionary methodsFor: 'private' stamp: 'monty 11/21/2014 01:43'!
deleteOrderedKey: aKey
	| index |

	index := self indexOfKey: aKey ifAbsent: [^ self].
	orderedKeys
		replaceFrom: index
		to: self size - 1
		with: orderedKeys
		startingAt: index + 1.
	orderedKeys at: self size put: nil.! !

!StandardOrderedDictionary methodsFor: 'private' stamp: 'monty 11/20/2014 18:07'!
dictionary
	^ dictionary! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:07'!
dictionaryClass
	^ Dictionary! !

!StandardOrderedDictionary methodsFor: 'enumerating' stamp: 'monty 11/20/2014 18:07'!
do: aBlock
	self valuesDo: aBlock! !

!StandardOrderedDictionary methodsFor: 'private' stamp: 'monty 11/21/2014 02:04'!
errorInvalidIndex: anIndex
	"squeak does not have SubscriptOutOfBounds."
	(Smalltalk
		at: #SubscriptOutOfBounds
		ifAbsent: [Error signal: 'Invalid index: ', anIndex printString]) signalFor: anIndex! !

!StandardOrderedDictionary methodsFor: 'private' stamp: 'monty 11/20/2014 18:07'!
growOrderedKeys
	orderedKeys :=
		(Array new: ((orderedKeys size * 1.5) asInteger max: 10))
			replaceFrom: 1
			to: orderedKeys size
			with: orderedKeys
			startingAt: 1.! !

!StandardOrderedDictionary methodsFor: 'comparing' stamp: 'monty 11/20/2014 18:07'!
hash
	^ dictionary hash! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:30'!
identityIndexOfKey: aKey ifAbsent: aBlock
	1 to: self size do: [:i |
		(orderedKeys at: i) == aKey
			ifTrue: [^ i]].
	^ aBlock value.! !

!StandardOrderedDictionary methodsFor: 'testing' stamp: 'monty 11/20/2014 18:07'!
includes: anObject
	^ dictionary includes: anObject! !

!StandardOrderedDictionary methodsFor: 'testing' stamp: 'monty 12/14/2014 17:03'!
includesAssociation: anAssociation
	"reimplemented for portability"
	^ (dictionary
		at: anAssociation key
		ifAbsent: [^ false]) = anAssociation value! !

!StandardOrderedDictionary methodsFor: 'testing' stamp: 'monty 12/14/2014 16:32'!
includesIdentity: anObject
	"reimplemented for portability"
	self valuesDo: [:each |
		each == anObject
			ifTrue: [^ true]].
	^ false.! !

!StandardOrderedDictionary methodsFor: 'testing' stamp: 'monty 11/20/2014 18:07'!
includesKey: aKey
	^ dictionary includesKey: aKey! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:07'!
indexOfKey: aKey
	^ self indexOfKey: aKey ifAbsent: [0]! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:10'!
indexOfKey: aKey ifAbsent: aBlock
	1 to: self size do: [:i |
		(orderedKeys at: i) = aKey
			ifTrue: [^ i]].
	^ aBlock value.! !

!StandardOrderedDictionary methodsFor: 'initialization' stamp: 'monty 11/20/2014 18:21'!
initialize: aCapacity
	dictionary := self dictionaryClass new: aCapacity.
	orderedKeys := Array new: aCapacity.! !

!StandardOrderedDictionary methodsFor: 'testing' stamp: 'monty 11/20/2014 18:10'!
isDictionary
	^ true! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/21/2014 01:38'!
keyAtIdentityValue: aValue
	^ dictionary keyAtIdentityValue: aValue! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 12/14/2014 16:32'!
keyAtIdentityValue: aValue ifAbsent: aBlock
	"reimplemented for portability"
	self keysAndValuesDo: [:key :value |
		value == aValue
			ifTrue: [^ key]].
	^ aBlock value.! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/21/2014 02:03'!
keyAtIndex: anIndex
	^ self keyAtIndex: anIndex ifAbsent: [self errorInvalidIndex: anIndex]! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:11'!
keyAtIndex: anIndex ifAbsent: aBlock
	(anIndex > 0 and: [anIndex <= self size])
		ifTrue: [^ orderedKeys at: anIndex]
		ifFalse: [^ aBlock value]! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/21/2014 01:39'!
keyAtValue: aValue
	^ dictionary keyAtValue: aValue! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:11'!
keyAtValue: aValue ifAbsent: aBlock
	^ dictionary keyAtValue: aValue ifAbsent: aBlock! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 12/14/2014 16:37'!
keyForIdentity: anObject
	"reimplemented for portability"
	self keysAndValuesDo: [:key :value |
		value == anObject
			ifTrue: [^ key]].
	^ nil.! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:11'!
keys
	^ orderedKeys copyFrom: 1 to: self size! !

!StandardOrderedDictionary methodsFor: 'enumerating' stamp: 'monty 11/20/2014 18:11'!
keysAndValuesDo: aBlock
	self keysDo: [:each | aBlock value: each value: (self at: each)]! !

!StandardOrderedDictionary methodsFor: 'removing' stamp: 'monty 11/20/2014 18:12'!
keysAndValuesRemove: aTwoArgumentBlock
	| removedAssociations |

	removedAssociations := OrderedCollection new.
	self associationsDo: [:each |
		(aTwoArgumentBlock value: each key value: each value)
			ifTrue: [removedAssociations add: each]].
	removedAssociations do: [:each | self removeKey: each key].! !

!StandardOrderedDictionary methodsFor: 'enumerating' stamp: 'monty 11/20/2014 18:11'!
keysDo: aBlock
	1 to: self size do: [:i | aBlock value: (orderedKeys at: i)]! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:12'!
keysSortedSafely
	^ dictionary keysSortedSafely! !

!StandardOrderedDictionary methodsFor: 'private' stamp: 'monty 11/20/2014 18:12'!
orderedKeys
	^ orderedKeys! !

!StandardOrderedDictionary methodsFor: 'copying' stamp: 'monty 11/20/2014 18:12'!
postCopy
	orderedKeys := orderedKeys copy.
	dictionary := dictionary copy.! !

!StandardOrderedDictionary methodsFor: 'printing' stamp: 'monty 11/21/2014 04:59'!
printElementsOn: aStream
	aStream nextPut: $(.
	self size > 100
		ifTrue: [
			aStream nextPutAll: 'size '.
			self size printOn: aStream]
		ifFalse: [
			self associations doWithIndex: [:associaiton :i |
				aStream
					print: associaiton key;
					nextPutAll: '->';
					print: associaiton value.
				(i < self size)
					ifTrue: [aStream space]]].
	aStream nextPut: $).! !

!StandardOrderedDictionary methodsFor: 'removing' stamp: 'monty 11/20/2014 18:19'!
remove: anObject ifAbsent: aBlock
	self shouldNotImplement! !

!StandardOrderedDictionary methodsFor: 'removing' stamp: 'monty 11/20/2014 18:12'!
removeAll
	1 to: self size do: [:i | orderedKeys at: i put: nil].
	dictionary removeAll.! !

!StandardOrderedDictionary methodsFor: 'removing' stamp: 'monty 11/21/2014 01:43'!
removeKey: aKey
	self deleteOrderedKey: aKey.
	^ dictionary removeKey: aKey.! !

!StandardOrderedDictionary methodsFor: 'removing' stamp: 'monty 11/21/2014 01:42'!
removeKey: aKey ifAbsent: aBlock
	self deleteOrderedKey: aKey.
	^ dictionary removeKey: aKey ifAbsent: aBlock.! !

!StandardOrderedDictionary methodsFor: 'enumerating' stamp: 'monty 11/20/2014 18:09'!
select: aBlock
	^ self speciesNewFrom:
		(self associations select: [:each | aBlock value: each value])! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:09'!
size
	^ dictionary size! !

!StandardOrderedDictionary methodsFor: 'private' stamp: 'monty 11/20/2014 18:51'!
speciesNewFrom: anAssociationCollection
	^ self species newFrom: anAssociationCollection! !

!StandardOrderedDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:08'!
values
	^ self associations collect: [:each | each value]! !

!StandardOrderedDictionary methodsFor: 'enumerating' stamp: 'monty 11/20/2014 18:08'!
valuesDo: aBlock
	self keysDo: [:each | aBlock value: (self at: each)]! !

StandardOrderedDictionary subclass: #StandardOrderedIdentityDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Collections-OrderPreservingDictionary'!

!StandardOrderedIdentityDictionary commentStamp: 'monty 11/21/2014 03:46' prior: 0!
An IdentityDictionary version that uses == instead of = for key comparing.!

!StandardOrderedIdentityDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 18:25'!
dictionaryClass
	^ IdentityDictionary! !

!StandardOrderedIdentityDictionary methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:35'!
indexOfKey: aKey ifAbsent: aBlock
	^ self identityIndexOfKey: aKey ifAbsent: aBlock! !
